/*
 * veraweb,
 * Veranstaltungsmanagment veraweb
 * Copyright (c) 2005-2007 tarent GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 * tarent GmbH., hereby disclaims all copyright
 * interest in the program 'veraweb'
 * Signature of Elmar Geese, 21 November 2007
 * Elmar Geese, CEO tarent GmbH.
 */

/*
 * $Id: FieldMapping.java,v 1.1 2007/06/20 11:56:52 christoph Exp $
 * 
 * Created on 22.08.2005
 */
package de.tarent.data.exchange;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Diese Klasse stellt eine Abbildung von Feldinhalten aufeinander dar.
 * 
 * @author mikel
 */
public class FieldMapping {
    //
    // Konstruktor
    //
    /**
     * Dieser Konstruktor bekommt die Menge der verfgbaren Quellfelder und
     * eine Abbildung Zielfeldname -> Formatstring mit Quellfeldreferenzen
     * bergeben. Quellfeldreferenzen haben allgemein die Form <b>{</b>BEREICH<b>:</b>FELD<b>}</b>,
     * im VerA.web-Fall insbesondere:<br>
     * <ul>
     * <li><b>{:</b>FELD<b>}</b> fr Stammdaten von Personen
     * <li><b>{CAT:</b>KATEGORIE<b>}</b> fr Kategoriezugehrigkeiten von Personen
     * <li><b>{EVE:</b>EREIGNIS<b>}</b> fr Ereigniszugehrigkeiten von Personen
     * <li><b>{COR:</b>DIPL. CORPS<b>}</b> fr Dipl.Corps-Zugehrigkeiten von Personen
     * <li><b>{DTM:</b>DOKUMENTTYP<b>}</b> fr Dokumenttypfreitexte von Hauptpersonen
     * <li><b>{DTP:</b>DOKUMENTTYP<b>}</b> fr Dokumenttypfreitexte von Partnerpersonen
     * </ul>
     * Sonderflle sind Benutzung von Jokern '*' im Bezeichner; hier ist nur eine
     * Quellfeldreferenz im Formatstring erlaubt, weiterhin muss im Zielfeld
     * ebenfalls ein Joker vorkommen. Solche Mappings werden interpretiert als
     * eine Zuordnung fr alle passenden bisher nicht zugeordneten Bezeichner
     * zu einem Zielfald, in dem der Joker durch den Bezeichnernamen ersetzt
     * wird.<br>
     * Im Falle konkurrierender Zuordnungen erhlt die speziellere den Vorrang.
     * 
     * @param availableSources verfgbare Quellen
     * @param mappingDescription Abbildung Zielfeldnamen auf Formatstrings mit
     *  Quellfeldreferenzen
     * @throws MappingException bei Problemen beim Auflsen der Mapping-Beschreibung
     * @throws NullPointerException falls ein Parameter <code>null</code> ist.
     */
    public FieldMapping(Set availableSources, Map mappingDescription) throws MappingException {
        this.availableSources = new HashSet(availableSources);
        this.mappingDescription = mappingDescription;
        parseDescription(mappingDescription);
    }
    
    /**
     * Dieser private Konstruktor wird von der Methode {@link #invert()} benutzt. 
     * 
     * @param availableSources verfgbare Quellen
     * @param availableTargets verfgbare Ziele
     * @param resolvedMapping aufgelstes (Joker-freies) Mapping
     */
    private FieldMapping(Set availableSources, Set availableTargets, Map resolvedMapping, Map mappingDescription) {
        if (availableTargets == null)
            availableTargets = resolvedMapping.keySet();
        else 
            assert resolvedMapping.keySet().containsAll(availableTargets);
        this.availableSources = new HashSet(availableSources);
        this.resolvedMappings = new HashMap(resolvedMapping);
        this.availableTargets = new HashSet(availableTargets);
        this.mappingDescription = new HashMap(mappingDescription);
    }
    
    //
    // Getter
    //
    /** verfgbare Quellfelder */
    public Set getSources() {
        return availableSources;
    }
    
    /** verfgbare Zielfelder */
    public Set getTargets() {
        return availableTargets;
    }
    
    //
    // ffentliche Interfaces
    //
    /**
     * Diese Schnittstelle stellt Quellentitten dar, aus deren Inhalten
     * {@link FieldMapping#resolve(String, Entity)} Zielfeldwerte erzeugt.
     */
    public interface Entity {
        /**
         * Diese Methode holt den Wert eines Quellfeldes bezglich eines
         * Quellfeldschlssels BEREICH<b>:</b>FELD.
         * 
         * @param sourceKey Quellfeldschlssel BEREICH<b>:</b>FELD 
         * @return Wert des beschriebenen Quellfelds
         * @see FieldMapping#FieldMapping(Set, Map)
         */
        public String get(String sourceKey);
    }
    
    //
    // ffentliche Methoden
    //
    /**
     * Diese Methode ermittelt den Wert eines Zielfelds basierend auf
     * den Werten einer Entitt.
     * 
     * @param targetKey Zielfeldbezeichner
     * @param entity Entitt, der die Quellfeldwerte zu entnehmen sind
     * @return ermittelter Zielfeldwert
     */
    public String resolve(String targetKey, Entity entity) {
        if (targetKey == null || entity == null || !resolvedMappings.containsKey(targetKey))
            return null;
        String format = resolvedMappings.get(targetKey).toString();
        Matcher matcher = refPattern.matcher(format);
        StringBuffer buffer = new StringBuffer();
        int from = 0;
        while (matcher.find()) {
            buffer.append(format.substring(from, matcher.start()))
                  .append(entity.get(matcher.group(1)));
            from = matcher.end();
        }
        if (from < format.length())
            buffer.append(format.substring(from));
        return buffer.toString();
    }
    
    /**
     * Gibt ein entpackten Wert entsprechend des bergeben Mappings an,
     * z.B. wird aus <code>Kat *</code> und <code>Kat Sample</code>
     * nur noch <code>Sample</code>. Beide Werte drfen nicht null sein.
     * 
     * Gibt null zurck wenn der String <code>value</code> nicht auf
     * <code>mapping</code matchet.
     * 
     * @param mapping
     * @param value
     * @return
     */
    public String resolve(String mapping, String value) {
    	int starlet = mapping.indexOf("*");
    	if (starlet == -1 || mapping.indexOf("*", starlet + 1) != -1)
    		return null;
    	
    	String pre = mapping.substring(0, starlet);
    	String suf = mapping.substring(starlet + 1);
    	
    	if (!(value.startsWith(pre) && value.endsWith(suf)))
    		return null;
    	
    	return value.substring(starlet, value.length() - suf.length());
    }
    
    /**
     * Diese Methode liefert ein Fieldmapping, das zu diesem invers ist.<br>
     * Hierbei sind gewisse Einschrnkungen zu beachten, deren Verletzung zu
     * ungeeigneten Mappings fhrt. Einerseits wird nur das erste Quellfeld
     * eines Mappings beachtet; weitere sollten also in einem weiteren Mapping
     * fhrend vorkommen. Andererseits sollte jedes Quellfeld nur einmal fhrend
     * vorkommen, um eine Eindeutigkeit zu gewhrleisten. 
     * 
     * @return ein Feldmapping, das zu diesem invers ist.
     */
    public FieldMapping invert() {
        assert resolvedMappings != null;
        Map invertedMapping = new HashMap();
        for (Iterator itMappings = resolvedMappings.entrySet().iterator(); itMappings.hasNext(); ) {
            Map.Entry mapping = (Entry) itMappings.next();
            Matcher matcher = refPattern.matcher(mapping.getValue().toString());
            if (matcher.find())
                invertedMapping.put(matcher.group(1), "{" + mapping.getKey() + '}');
        }
        return new FieldMapping(availableTargets, invertedMapping.keySet(), invertedMapping, mappingDescription);
    }

    /**
     * Diese Methode erweitert das FieldMapping um Kategorien-Felder die im
     * Header bergeben wurde, aber noch nicht in der Datenbank vorhanden sind
     * und deswegen nicht in den {@link #availableSources} verfgbar waren.
     * 
     * @param headers
     */
	public void extendCategoryImport(List extensions) {
		assert mappingDescription != null && resolvedMappings != null;
		
        // Map der aufgelsten Mappings
        Map finalMappings = new HashMap();
        finalMappings.putAll(resolvedMappings);
        
		for (Iterator it = mappingDescription.entrySet().iterator(); it.hasNext(); ) {
			Map.Entry entry = (Map.Entry)it.next();
			String mappingMarker = entry.getKey().toString();
			String mappingTarget = entry.getValue().toString();
			
			if (mappingTarget == null || mappingMarker == null)
				continue;
			if (mappingTarget.startsWith("{") || mappingTarget.startsWith("}"))
				mappingTarget = mappingTarget.substring(1, mappingTarget.length() - 1);
			if (!(
					mappingTarget.startsWith("CAT:") ||
					mappingTarget.startsWith("EVE:") ||
					mappingTarget.startsWith("COR:")))
				continue;
			
			for (Iterator it2 = extensions.iterator(); it2.hasNext(); ) {
				String extension = it2.next().toString();
				String cleaned = resolve(mappingMarker, extension);
				if (cleaned == null) continue;
				String resolved = mappingTarget.replaceAll("\\*", cleaned);
				if (resolved == null) continue;
				
				finalMappings.put(resolved, "{" + extension + "}");
			}
		}
		
		resolvedMappings = Collections.unmodifiableMap(finalMappings);
		availableTargets = finalMappings.keySet();
	}

	//
    // geschtzte Hilfsmethoden
    //
    /**
	 * Diese Methode ermittelt mittels der verfgbaren Quellfelder und der
	 * allgemeinen Abbildungsbeschreibung (die im Konstruktor
	 * {@link #FieldMapping(Set, Map)} bergeben wurden) die aufgelsten simplen
	 * Abbildungen und die verfgbaren Zielfelder.
	 * 
	 * @param mappingDescription
	 *            die zu parsende Abbildungsbeschreibung
	 * @throws MappingException
	 *             bei Problemen beim Auflsen der Mapping-Beschreibung
	 */
    void parseDescription(Map mappingDescription) throws MappingException {
        assert availableSources != null;
        assert mappingDescription != null;
        // Menge der (bisher) unbenutzten Quellen --- wird fr Joker-Auflsung genutzt
        Set unusedSources = new HashSet(availableSources);
        // Map Prfixlnge vor Joker in Quellbezeichnern (Integer) -> Jokerzielfeldbezeichner --- wird fr Joker-Auflsung genutzt
        Map targetsByPrefixLength = new HashMap();
        // Zielfeldbezeichner in targetsByPrefixLength einsortieren
        for (Iterator itMappingEntries = mappingDescription.entrySet().iterator(); itMappingEntries.hasNext(); ) {
            Map.Entry entry = (Entry) itMappingEntries.next();
            String targetFieldKey = entry.getKey().toString();
            int jokerIndex = removeReferences(unusedSources, entry.getValue().toString());
            Integer prefixLength = new Integer(jokerIndex);
            Collection theseTargets = (Collection) targetsByPrefixLength.get(prefixLength);
            if (theseTargets == null)
                targetsByPrefixLength.put(prefixLength, theseTargets = new HashSet());
            theseTargets.add(targetFieldKey);
        }
        // Map der aufgelsten Mappings
        Map finalMappings = new HashMap();
        // Einfache (Joker-freie) Mappings bertragen
        if (targetsByPrefixLength.containsKey(MINUS_ONE)) {
            for (Iterator itSimpleMappingTargets = ((Collection)targetsByPrefixLength.get(MINUS_ONE)).iterator(); itSimpleMappingTargets.hasNext(); ) {
                String targetKey = itSimpleMappingTargets.next().toString();
                finalMappings.put(targetKey, mappingDescription.get(targetKey));
            }
            targetsByPrefixLength.remove(MINUS_ONE);
        }
        // Komplexe (Joker-behaftete) Mappings auflsen und bertragen
        if (targetsByPrefixLength.size() > 0) {
            Integer[] lengths = (Integer[]) targetsByPrefixLength.keySet().toArray(new Integer[targetsByPrefixLength.size()]);
            Arrays.sort(lengths);
            for (int index = lengths.length - 1; index >= 0; index--) {
                for (Iterator itMappingTargets = ((Collection)targetsByPrefixLength.get(lengths[index])).iterator(); itMappingTargets.hasNext(); ) {
                    String targetKey = itMappingTargets.next().toString();
                    String format = mappingDescription.get(targetKey).toString();
                    String jokerReferencePrefix = getJokerReferencePrefix(format);
                    Set toRemove = new HashSet();
                    for (Iterator itUnusedSources = unusedSources.iterator(); itUnusedSources.hasNext(); ) {
                        String source = itUnusedSources.next().toString();
                        if (source.startsWith(jokerReferencePrefix)) {
                            addResolvedMapping(finalMappings, targetKey, format, source);
                            toRemove.add(source);
                        }
                    }
                    unusedSources.removeAll(toRemove);
                }
                if (unusedSources.isEmpty())
                    break;
            }
        }
        resolvedMappings = Collections.unmodifiableMap(finalMappings);
        availableTargets = finalMappings.keySet();
    }
    
    /**
     * Diese Methode sucht zu einem Formatstring eine Referenz mit Joker
     * und liefert hierzu das Anfangsstck zurck.
     * 
     * @param format dieser Formatstring wird auf Referenzen mit Joker durchsucht.
     * @return Anfangsstck der Referenz bis exklusive dem Joker, sonst <code>null</code>.
     */
    private static String getJokerReferencePrefix(String format) {
        assert format != null;
        Matcher matcher = refPattern.matcher(format);
        while (matcher.find()) {
            String ref = matcher.group(1);
            int jokerIndex = ref.indexOf('*');
            if (jokerIndex >= 0)
                return ref.substring(0, jokerIndex);
        }
        return null;
    }
    
    /**
     * Diese Methode hat eine doppelte Funktion; einerseits entfernt sie alle einfachen
     * (also nicht Joker-behafteten) Referenzen im Formatstring aus der Sammlung nicht
     * benutzter Referenzen, und andererseits liefert sie die Joker-Position in einer
     * gegebenenfalls vorhandenen Joker-behafteten Referenz zurck.
     * 
     * @param unusedReferences Sammlung bisher nicht benutzter verfgbarere Referenzen
     * @param format zu behandelnder Formatstring
     * @return Joker-Position in der vorhandenen Joker-behafteten Referenz, sonst <code>-1</code>.
     * @throws MappingException bei Problemen beim Auflsen der Mapping-Beschreibung
     */
    private static int removeReferences(Collection unusedReferences, String format) throws MappingException {
        int jokerPrefixLength = -1;
        Matcher matcher = refPattern.matcher(format);
        while (matcher.find()) {
            String ref = matcher.group(1);
            int jokerIndex = ref.indexOf('*');
            if (jokerIndex < 0)
                unusedReferences.remove(ref);
            else if (jokerIndex != ref.length() - 1) 
                throw new MappingException("es werden nur endstndige Joker im Formatstring untersttzt");
            else if (jokerPrefixLength < 0)
                jokerPrefixLength = jokerIndex;
            else
                throw new MappingException("nur eine Referenz in einem Formatstring darf Joker enthalten");
        }
        return jokerPrefixLength;
    }
    
    /**
     * Diese Methode fgt den einfachen Mappings eines hinzu, das dem bergebenen mit
     * Joker behafteten nach Ersetzung mittels der bergebenen Quelle entspricht. 
     * 
     * @param resolvedMappings {@link Map} der aufgelsten einfachen Mappings
     * @param targetPattern Zielreferenz mit Joker
     * @param formatPattern Formatstring mit Joker-behafteter Quellreferenz
     * @param source Quellfeld zur Auflsung der Joker.
     */
    private static void addResolvedMapping(Map resolvedMappings, String targetPattern, String formatPattern, String source) {
        assert formatPattern != null;
        Matcher matcher = refPattern.matcher(formatPattern);
        while (matcher.find()) {
            String ref = matcher.group(1);
            int jokerIndex = ref.indexOf('*');
            if (jokerIndex >= 0) {
                StringBuffer buffer = new StringBuffer();
                buffer.append(formatPattern.substring(0, matcher.start(1)))
                      .append(source)
                      .append(formatPattern.substring(matcher.end(1)));
                int colonIndex = source.indexOf(':');
                if (colonIndex >= 0)
                    source = source.substring(colonIndex + 1);
                targetPattern = targetPattern.replaceAll("\\*", source);
                Object former = resolvedMappings.put(targetPattern, buffer.toString());
                if (former != null)
                    logger.warning("Ersetze altes Format '" + former + "' fr '" + targetPattern + "' durch '" + buffer.toString() + "'.");
                return;
            }
        }
    }
    
    //
    // geschtzte Member
    //
    final Set availableSources;
    final Map mappingDescription;
    Map resolvedMappings = null;
    Set availableTargets = null;
    final static Pattern refPattern = Pattern.compile("\\{([^{}]*)\\}");
    final static Integer MINUS_ONE = new Integer(-1);
    final static Logger logger = Logger.getLogger(FieldMapping.class.getName());
}
